package com.capitalware.ufm.action.objects;

import java.io.File;
import java.util.ArrayList;

import com.capitalware.ufm.UFM;
import com.capitalware.ufm.handlers.StringUtilities;
import com.capitalware.ufm.handlers.Tokens;
import com.capitalware.ufm.objects.CWConstants;

/**
 * This class holds the values for the Touch action.
 *
 * @author Roger Lacroix
 * @version 1.0
 * @license Apache 2 License
 */
public class TouchObject extends UFMFileObject
{
   public String[]   filenames = null;
   public boolean    createDir = false;

   /**
    * The constructor
    */
   public TouchObject()
   {
      super();
   }

   /**
    * Process tokens in Action Object's variables
    */
   public void performTokenization()
   {
      for (int i=0; i < filenames.length; i++)
         filenames[i] = Tokens.replace(filenames[i], paClass.actionProperties);
   }

   /**
    * Verify the values in the object.
    */
   public void checkObject()
   {
      if (filenames == null)
         throw new IllegalArgumentException("Error: You must specify a filename for the Touch action.");
      
      for (int i=0; i < filenames.length; i++)
      {
         if (filenames[i].indexOf(File.separator) != -1)
            checkAndCreateDirectory(createDir, filenames[i].substring(0,filenames[i].lastIndexOf(File.separator)));
      }
   }

   /**
    * Output the info
    * @param logger
    */
   public void printValues()
   {
      UFM.logger.info("     FileName             = " + filenames[0]);
      for (int i=1; i < filenames.length; i++)
         UFM.logger.info("                            " + filenames[i]);
      UFM.logger.info("     Create Directory     = " + createDir);
   }
   
   /**
    * Convert the object to a string.
    * @return s
    */
   public ArrayList<String> toStringArray()
   {
      ArrayList<String> al = new ArrayList<String>();

      al.add("<"+CWConstants.TOUCH_ELEMENT+">"+LINE_SEPARATOR);

      for (int i=0; i < filenames.length; i++)
      {
         if (filenames[i].indexOf(File.separator) != -1 )
         {
            int x = filenames[i].lastIndexOf(File.separator);

            String dir   = filenames[i].substring(0, x);
            String fname = filenames[i].substring(x+1);

            al.add("  <"+CWConstants.FILE_ELEMENT+" "+CWConstants.DIR_ATTRIBUTE+"=\""+dir+"\">"+fname+"</"+CWConstants.FILE_ELEMENT+">"+LINE_SEPARATOR);
         }
         else
            al.add("  <"+CWConstants.FILE_ELEMENT+">"+filenames[i]+"</"+CWConstants.FILE_ELEMENT+">"+LINE_SEPARATOR);
      }
      
      al.add("</"+CWConstants.TOUCH_ELEMENT+">"+LINE_SEPARATOR);
      
      return al;
   }

   /**
    * Retrieve the values in this action for the status update.
    * @return s
    */
   public String toStringForStatusUpdate(String event)
   {
      String s;

      s = CWConstants.TOUCH_ELEMENT+", "+labelName+", "+CWConstants.TOUCH_ELEMENT+", "+CWConstants.CREATE_DIR_ATTRIBUTE+"="+createDir+", filenames="+StringUtilities.toStringFromArrayOfString(filenames)+LINE_SEPARATOR;
      
      return s;
   }
}
