package com.capitalware.ufm.actions;

import com.capitalware.ufm.UFM;
import com.capitalware.ufm.action.objects.AppendObject;
import com.capitalware.ufm.handlers.IO;
import com.capitalware.ufm.handlers.Tokens;
import com.capitalware.ufm.objects.UFMException;

/**
 * This class will handle the append action.
 *
 * @author Roger Lacroix
 * @version 1.0
 * @license Apache 2 License
 */
public class AppendAction extends UFMAction
{
   private AppendObject   ao;

   /**
    * The constructor
    * @param ao
    */
   public AppendAction(AppendObject ao)
   {
      super(ao.actionName);
      this.ao = ao;
   }

   /**
    * Perform the copy work.
    */
   public int doWork()
   {
      int exitCode = 0;
      String toPathAndFilename;
      ao.printValues();

      toPathAndFilename = IO.apendFileToPath(ao.toDir, ao.toFile);
      toPathAndFilename = Tokens.replace(ao.actionName, toPathAndFilename, IO.getFileName(ao.srcFilename), 1, ao.paClass.actionProperties);
      
      try
      {
         IO.appendFile(ao.srcFilename, toPathAndFilename);
         ao.fileCount++;
      }
      catch (UFMException e)
      {
         UFM.logger.fatal("UFMException: "+e.getLocalizedMessage());
         if (e.getCause() != null)
            UFM.logger.fatal(e.getCause().toString());;
         exitCode = 8;
      }
      finally
      {
         if (!ao.onErrorFail)
            exitCode = 0; /* reset back to zero */
      }
      
      return exitCode;
   }
}
