package com.capitalware.ufm.actions;

import java.io.File;

import com.capitalware.ufm.UFM;
import com.capitalware.ufm.action.objects.MakeDirObject;

/**
 * This class will handle the MakeDir action.
 *
 * @author Roger Lacroix
 * @version 1.0
 * @license Apache 2 License
 */
public class MakeDirAction extends UFMAction
{
   private MakeDirObject   mdo;

   /**
    * The constructor
    * @param mdo
    */
   public MakeDirAction(MakeDirObject mdo)
   {
      super(mdo.actionName);
      this.mdo = mdo;
   }

   /**
    * Perform the MakeDir work.
    * @return exitCode
    */
   public int doWork()
   {
      int exitCode = 0;
      mdo.printValues();

      if ((new File(mdo.targetDir)).exists())
      {
         UFM.logger.warn("Directory already exists: " + mdo.targetDir);
      }
      else
      {
         if ((new File(mdo.targetDir)).mkdirs()) 
         {
            UFM.logger.info("Successfully created directory: " + mdo.targetDir);
         }
         else
         {
            UFM.logger.fatal("Cannot create directory: " + mdo.targetDir);
            UFM.globalErrorText = "Cannot create directory: " + mdo.targetDir;
            exitCode = 8;
         }
      }
 
      if (!mdo.onErrorFail)
         exitCode = 0; /* reset back to zero */

      return exitCode;
   }
}
