package com.capitalware.ufm.actions;

import java.io.File;
import java.io.IOException;
import java.util.List;

import com.capitalware.ufm.UFM;
import com.capitalware.ufm.action.objects.MergeSortObject;
import com.capitalware.ufm.handlers.ExternalSort;
import com.capitalware.ufm.handlers.ExternalSortComparator;
import com.capitalware.ufm.handlers.IO;
import com.capitalware.ufm.handlers.Tokens;
import com.capitalware.ufm.objects.UFMException;

/**
 * This class will handle the MergeSort action.
 * 
 * @author Roger Lacroix
 * @version 1.0
 * @license Apache 2 License
 */
public class MergeSortAction extends UFMAction
{
   private MergeSortObject mso;

   /**
    * The constructor
    * @param mso
    */
   public MergeSortAction(MergeSortObject mso)
   {
      super(mso.actionName);
      this.mso = mso;
   }

   /**
    * Perform the work.
    * @return exitCode
    */
   public int doWork()
   {
      int exitCode = 0;
      String toPathAndFilename;
      File tempFile;
      mso.printValues();

      toPathAndFilename = IO.apendFileToPath(mso.toDir, mso.toFile);
      toPathAndFilename = Tokens.replace(mso.actionName, toPathAndFilename, IO.getFileName(mso.srcFilenames[0]), 1, mso.paClass.actionProperties);

      try
      {
         mso.srcFilenames = IO.buildStringArrayOfFiles(mso.srcFilenames);
         
         tempFile = File.createTempFile(com.capitalware.ufm.UFM.WHO_AM_I+"_"+mso.actionName+"_", ".tmp");
         
         // Delete temp file when program exits.
         tempFile.deleteOnExit();      
         
         IO.mergeFiles(mso.srcFilenames, tempFile.getAbsolutePath());

         ExternalSortComparator esc = new ExternalSortComparator(mso);
         List<File> l = ExternalSort.sortInBatch(new File(tempFile.getAbsolutePath()), esc) ;
         ExternalSort.mergeSortedFiles(l, new File(toPathAndFilename), esc);
         
         if (!(tempFile.delete()))
            UFM.logger.fatal("Error: Cannot delete temporary file: "+tempFile.getAbsolutePath());
         
         mso.fileCount++;

         UFM.logger.info("Successfully merged and sortd files into " + toPathAndFilename);
      }
      catch (IOException e)
      {
         UFM.logger.fatal("IOException: " + e.getLocalizedMessage());
         UFM.globalErrorText = e.getLocalizedMessage();
         exitCode = 8;
      }
      catch (UFMException e)
      {
         UFM.logger.fatal("UFMException: "+e.getLocalizedMessage());
         if (e.getCause() != null)
            UFM.logger.fatal(e.getCause().toString());;
         exitCode = 8;
      }
      finally
      {
         if (!mso.onErrorFail)
            exitCode = 0; /* reset back to zero */
      }

      return exitCode;
   }
}
