package com.capitalware.ufm.actions;

import java.io.File;
import java.io.IOException;

import com.capitalware.ufm.UFM;
import com.capitalware.ufm.action.objects.ReEncodeObject;
import com.capitalware.ufm.handlers.IO;
import com.capitalware.ufm.handlers.Tokens;
import com.capitalware.ufm.objects.UFMException;

/**
 * This class will handle the ReEncode action.
 *
 * @author Roger Lacroix
 * @version 1.0
 * @license Apache 2 License
 */
public class ReEncodeAction extends UFMAction
{
   private ReEncodeObject   reo;

   /**
    * The constructor
    * @param reo
    */
   public ReEncodeAction(ReEncodeObject reo)
   {
      super(reo.actionName);
      this.reo = reo;
   }

   /**
    * Perform the convert work.
    * @return exitCode
    */
   public int doWork()
   {
      int exitCode = 0;
      reo.printValues();

      if (reo.srcFilename.indexOf('*') != -1)
         exitCode = handleManyFile();
      else
         exitCode = handleOneFile();

      if (!reo.onErrorFail)
         exitCode = 0; /* reset back to zero */
      
      return exitCode;
   }

   /**
    * ReEncode a single file
    * @return exitCode
    */
   private int handleOneFile()
   {
      int exitCode = 0;
      String toPathAndFilename;
      String toFilename;
      
      if (reo.toFile != null)
         toFilename = IO.getFileName(reo.toFile);
      else
         toFilename = IO.getFileName(reo.srcFilename);

      toPathAndFilename = IO.apendFileToPath(reo.toDir, toFilename);

      toPathAndFilename = Tokens.replace(reo.actionName, toPathAndFilename, IO.getFileName(reo.srcFilename), 1, reo.paClass.actionProperties);

      try
      {
         IO.reencodeFile(reo.srcFilename, reo.inputEncoding, toPathAndFilename, reo.outputEncoding);
         reo.fileCount++;
      }
      catch (UFMException e)
      {
         exitCode = 8;
      }

      return exitCode;
   }

   /**
    * ReEncode many files
    * @return exitCode
    */
   private int handleManyFile()
   {
      int exitCode = 0;
      String toPathAndFilename;
      File[] fileList;
      
      try
      {
         fileList = IO.getFilteredFileList(reo.srcFilename);
         
         for (int i=0; i < fileList.length; i++)
         {
            String toFilename;

            if (reo.toFile == null)
               toFilename = fileList[i].getName();
            else
               toFilename = reo.toFile;

            toPathAndFilename = IO.apendFileToPath(reo.toDir, toFilename);

            toPathAndFilename = Tokens.replace(reo.actionName, toPathAndFilename, fileList[i].getName(), i+1, reo.paClass.actionProperties);

            try
            {
               IO.reencodeFile(fileList[i].toString(), reo.inputEncoding, toPathAndFilename, reo.outputEncoding);
               reo.fileCount++;
            }
            catch (UFMException e)
            {
               exitCode = 8;
            }
         }
      }
      catch (IOException e)
      {
         UFM.logger.fatal("IOException: " + e.getLocalizedMessage());
         UFM.globalErrorText = e.getLocalizedMessage();
         exitCode = 8;
      }

      return exitCode;
   }

}
