package com.capitalware.ufm.actions;

import com.capitalware.ufm.UFM;
import com.capitalware.ufm.action.objects.WriteTextObject;
import com.capitalware.ufm.handlers.IO;
import com.capitalware.ufm.handlers.Tokens;
import com.capitalware.ufm.objects.UFMException;

/**
 * This class will handle the WriteText action.
 * 
 * @author Roger Lacroix
 * @version 1.0
 * @license Apache 2 License
 */
public class WriteTextAction extends UFMAction
{
   private WriteTextObject wto;

   /**
    * The constructor
    * @param mo
    */
   public WriteTextAction(WriteTextObject wto)
   {
      super(wto.actionName);
      this.wto = wto;
   }

   /**
    * Perform the work.
    * @return exitCode
    */
   public int doWork()
   {
      int exitCode = 0;
      String toPathAndFilename;
      wto.printValues();

      toPathAndFilename = IO.apendFileToPath(wto.toDir, wto.toFile);
      toPathAndFilename = Tokens.replace(wto.actionName, toPathAndFilename, IO.getFileName(wto.toFile), 1, wto.paClass.actionProperties);

      try
      {
         IO.writeFile(toPathAndFilename, wto.text.getBytes(), wto.append);
      }
      catch (UFMException e)
      {
         if (!wto.onErrorFail)
            UFM.handleOnError(wto.paClass.actionProperties, null);
         
         UFM.logger.fatal("UFMException: "+e.getLocalizedMessage());
         if (e.getCause() != null)
            UFM.logger.fatal(e.getCause().toString());;
         exitCode = 8;
      }
      finally
      {
         if (!wto.onErrorFail)
            exitCode = 0; /* reset back to zero */
      }

      return exitCode;
   }
}
