package com.capitalware.ufm.handlers;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.UnsupportedEncodingException;
import java.security.InvalidAlgorithmParameterException;
import java.security.InvalidKeyException;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;

import javax.crypto.BadPaddingException;
import javax.crypto.Cipher;
import javax.crypto.IllegalBlockSizeException;
import javax.crypto.NoSuchPaddingException;
import javax.crypto.spec.IvParameterSpec;
import javax.crypto.CipherInputStream;
import javax.crypto.CipherOutputStream;

import java.security.spec.AlgorithmParameterSpec;
import javax.crypto.spec.SecretKeySpec;

public class AES
{
   private static final String NUMBER_PAD = "12345678901234567890123456789012";

   // Buffer used to transport the bytes from one stream to another
   private byte[] streamBuffer = new byte[4096];
   
   private Cipher ecipher;
   private Cipher dcipher;

   /**
    * Input a string that will be md5 hashed to create the key.
    * 
    * @return void, cipher initialized
    * @throws NoSuchPaddingException 
    * @throws NoSuchAlgorithmException 
    * @throws InvalidAlgorithmParameterException 
    * @throws InvalidKeyException 
    * @throws UnsupportedEncodingException 
    */
   public AES(String passPhrase, int keySize) throws NoSuchAlgorithmException, NoSuchPaddingException, InvalidKeyException, InvalidAlgorithmParameterException, UnsupportedEncodingException
   {
      super();
      
      int ppLen = keySize/8;
      String pp = passPhrase + NUMBER_PAD;  // pad out the passphrase
      byte[] hash = new byte[ppLen];

      /*
       * Get the hash for the lower 16 bytes - common to all 3 keySizes
       */
      byte[] h1 = getMD5(pp.substring(0,16));
     
      for (int i=0; i < 16; i++)
         hash[i] = h1[i];
      
      if (keySize == 128)
      {
         // Already handled
      }
      else if (keySize == 192)
      {
         byte[] h2 = getMD5(pp.substring(16,24));
         
         for (int i=0; i < 8; i++)
            hash[16+i] = h2[i];
      }
      else if (keySize == 256)
      {
         byte[] h2 = getMD5(pp.substring(16,32));
         
         for (int i=0; i < 16; i++)
            hash[16+i] = h2[i];
      }
      
      SecretKeySpec skey = new SecretKeySpec(hash, "AES");

      // Create an 8-byte initialization vector
      byte[] iv = new byte[] { 0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07,
                               0x08, 0x09, 0x0a, 0x0b, 0x0c, 0x0d, 0x0e, 0x0f };

      AlgorithmParameterSpec paramSpec = new IvParameterSpec(iv);
      
      ecipher = Cipher.getInstance("AES/CBC/PKCS5Padding");
      dcipher = Cipher.getInstance("AES/CBC/PKCS5Padding");

      // CBC requires an initialization vector
      ecipher.init(Cipher.ENCRYPT_MODE, skey, paramSpec);
      dcipher.init(Cipher.DECRYPT_MODE, skey, paramSpec);
   }

   /**
    * Create an MD5 
    * @param input
    * @return digest
    * @throws UnsupportedEncodingException 
    * @throws NoSuchAlgorithmException 
    */
   private byte[] getMD5(String input) throws UnsupportedEncodingException, NoSuchAlgorithmException
   {
      byte[] bytesOfMessage = input.getBytes("UTF-8");
      MessageDigest md = MessageDigest.getInstance("MD5");
      return md.digest(bytesOfMessage);
   }

   /**
    * Encrypt data on a stream to another stream.
    * @param in
    * @param out
    * @throws IOException 
    */
   public void encrypt(InputStream in, OutputStream out) throws IOException
   {
      int numRead = 0;
      CipherOutputStream outCOS = null;

      try
      {
         // Bytes written to out will be encrypted
         outCOS = new CipherOutputStream(out, ecipher);
   
         // Read in the cleartext bytes and write to out to encrypt
         while ((numRead = in.read(streamBuffer)) >= 0)
         {
            outCOS.write(streamBuffer, 0, numRead);
         }
      }
      finally
      {
         try { if (outCOS != null) outCOS.close(); } catch (Exception e) {;}
         try { if (in != null)     in.close(); } catch (Exception e) {;}
         try { if (out != null)    out.close(); } catch (Exception e) {;}
      }
   }

   /**
    * Input is a string to encrypt.
    * 
    * @return a Hex string of the byte array
    * @throws UnsupportedEncodingException 
    * @throws BadPaddingException 
    * @throws IllegalBlockSizeException 
    */
   public byte[] encrypt(String plaintext) throws IllegalBlockSizeException, BadPaddingException, UnsupportedEncodingException
   {
      return ecipher.doFinal(plaintext.getBytes("UTF-8"));
   }

   /**
    * Input is a string to encrypt.
    * 
    * @return a Hex string of the byte array
    * @throws encrypted array of bytes 
    * @throws BadPaddingException 
    * @throws IllegalBlockSizeException 
    */
   public byte[] encrypt(byte[] plaintext) throws IllegalBlockSizeException, BadPaddingException, UnsupportedEncodingException
   {
      return ecipher.doFinal(plaintext);
   }

   /**
    * Decrypt data on a stream to another stream.
    * @param in
    * @param out
    * @throws IOException 
    */
   public void decrypt(InputStream in, OutputStream out) throws IOException
   {
      int numRead = 0;
      CipherInputStream inCIS = null;

      try
      {
         // Bytes read from in will be decrypted
         inCIS = new CipherInputStream(in, dcipher);
   
         // Read in the decrypted bytes and write the cleartext to out
         while ((numRead = inCIS.read(streamBuffer)) >= 0)
         {
            out.write(streamBuffer, 0, numRead);
         }
      }
      finally
      {
         try { if (inCIS != null) inCIS.close(); } catch (Exception e) {;}
         try { if (in != null)    in.close(); } catch (Exception e) {;}
         try { if (out != null)   out.close(); } catch (Exception e) {;}
      }
   }

   /**
    * Decrypt the array of bytes
    * @param ciphertext
    * @return
    * @throws BadPaddingException 
    * @throws IllegalBlockSizeException 
    */
   public byte[] decrypt(byte[] ciphertext) throws IllegalBlockSizeException, BadPaddingException
   {
      return dcipher.doFinal(ciphertext);
   }
}
