package com.capitalware.ufm.mq;

import java.io.IOException;

import com.capitalware.ufm.UFM;
import com.capitalware.ufm.action.objects.MQPutQuitObject;
import com.capitalware.ufm.mq.objects.MQObject;
import com.capitalware.ufm.objects.CWConstants;
import com.capitalware.ufm.objects.UFMException;

import com.ibm.mq.*;
import com.ibm.mq.constants.CMQC;

/**
 * This class will put a 'Quit' message on a queue.
 *
 * @author Roger Lacroix
 * @version 1.0
 * @license Apache 2 License
 */
public class MQPutQuit extends MyMQ
{
   /**
    * The constructor
    */
   public MQPutQuit(MQObject qm)
   {
   	super(qm);
   }

   /**
    * Put a 'QUIT' message to a queue.
    *
    * @param queueName
    * @throws UFMException 
    */
   public void simplePut1(MQPutQuitObject  pqo) throws UFMException
   {
      MQQueue _outQ = null;
      MQPutMessageOptions pmo = new MQPutMessageOptions();
      pmo.options = CMQC.MQPMO_FAIL_IF_QUIESCING;

      try
      {
         _outQ = openOutQ(mqo.qObject, false);

         UFM.logger.info("Successfully opened queue: " + mqo.qObject);
         MQMessage sendmsg = new MQMessage();

         sendmsg.messageId     = CMQC.MQMI_NONE;
         sendmsg.correlationId = CMQC.MQCI_NONE;
         sendmsg.feedback      = CMQC.MQFB_QUIT;
         sendmsg.format        = CMQC.MQFMT_STRING;
         sendmsg.messageType   = CMQC.MQMT_DATAGRAM;

         sendmsg.write(CWConstants.QUIT_MSG.getBytes());

         // put the message on the queue.
         _outQ.put(sendmsg, pmo);

         UFM.logger.info("Put '"+ CWConstants.QUIT_MSG +"' message to " + mqo.qObject  );
      }
      catch (MQException e)
      {
         UFM.logger.fatal("MQException: " + e.getMessage() + " : QName=" + mqo.qObject);
         UFM.logger.fatal("MQCC=" + e.completionCode + " : MQRC="+e.reasonCode);
         UFM.globalErrorText = e.getLocalizedMessage();
      }
      catch (IOException ioex)
      {
         UFM.logger.fatal("IOException: " + ioex.getMessage());
         UFM.globalErrorText = ioex.getLocalizedMessage();
      }
      finally
      {
         try
         {
            // Done. Close the queue.
            if (_outQ != null)
            {
               _outQ.close();
               UFM.logger.info("Successfully closed the queue.");
            }
         }
         catch (MQException e)
         {
            UFM.logger.fatal("MQException: " + e.getMessage() + " : QName=" + mqo.qObject);
            UFM.logger.fatal("MQCC=" + e.completionCode + " : MQRC="+e.reasonCode);
            UFM.globalErrorText = e.getLocalizedMessage();
         }
      }
   }
}
