package com.capitalware.ufm.mq.objects;

import java.io.File;
import java.util.ArrayList;

import com.capitalware.ufm.UFM;

/**
 * This class holds the MQ information.
 * @author Roger Lacroix, Capitalware Inc.
 * @version 1.1
 * @license Apache 2 License
 */
public class MQObject
{
   public String                       qMgrName = null;
   public QueueObject                  qObject = null;
   public ArrayList<QueueObject>       disListOfQNames = null;
   public String                       hostname = null;
   public String                       channelName = null;
   public int                          port = -1;
   //
   public String                       userID = System.getProperty("user.name");
   public String                       password = null;
   //
   public String                       securityExit = null;
   public String                       securityExitPath = null;
   public String                       securityExitData = null;
   //
   public String                       cipherSuiteName = null;
   public String                       distinguishedName = null;
   public String                       trustedStore = null;
   public String                       trustedStorePasswd = null;
   public String                       keyStore = null;
   public String                       keyStorePasswd = null;
   public String                       ldapServer = null;
   public int                          ldapServerPort = -1;
   //
   public String                       CCDTFile = null;
   public String                       mqXmlFile = null;

   /*
    * The constructor
    */
   public MQObject(String qMgrName, QueueObject qObject, String CCDTFile)
   {
      super();
      this.qMgrName = qMgrName;
      this.qObject = qObject;
      this.CCDTFile = CCDTFile;
   }

   /*
    * The constructor
    */
   public MQObject(String qMgrName, QueueObject qObject, String mqXmlFile, String CCDTFile)
   {
      super();
      this.qMgrName = qMgrName;
      this.qObject = qObject;
      this.mqXmlFile = mqXmlFile;
      this.CCDTFile = CCDTFile;
   }

   /**
    * Verify the values in the MQOject
    */
   public void checkQM()
   {
      if ( ("".equals(qMgrName)) || (qMgrName == null) )
         throw new IllegalArgumentException("Missing a required XML tag: QMgrName within MQ");

      if ( (qObject == null) && (disListOfQNames == null) )
         throw new IllegalArgumentException("Missing value. Specify either a queue name or a distribution list.");

      if ( ((qObject == null) || ("".equals(qObject.qName))) && (disListOfQNames == null) )
         throw new IllegalArgumentException("Missing a required XML tag: QueueName within MQ");
      
      if ( (userID == null) && (password != null) )
         throw new IllegalArgumentException("Cannot have a Password without a UserID: UserID within MQ");

      if ((hostname != null) || (channelName != null))
      {
         if (hostname == null)
            throw new IllegalArgumentException("Missing the XML tag: Hostname within UFM_MQ.");
         if (channelName == null)
            throw new IllegalArgumentException("Missing the XML tag: ChannelName within UFM_MQ.");

         if (CCDTFile != null)
            throw new IllegalArgumentException("The user cannot specify both channelname/hostname/port and CCDTFile.  Please specify only one method of connecting to the remote queue manager.");
      }
      else if (CCDTFile != null)
      {
         boolean exists = (new File(CCDTFile)).exists();
         if ( !(exists) )
            throw new IllegalArgumentException("Error: The CCDT file does not exists : " + CCDTFile);
      }
   }

   /**
    * Dump the common user supplied parameters.
   * @param logger
    */
   public void dumpUserParms()
   {
      UFM.logger.info("     Queue Manager Name   = " + qMgrName);
      
      if (disListOfQNames != null)
      {
         StringBuffer sb = new StringBuffer();
         for (int i=0; i < disListOfQNames.size(); i++)
         {
            if (i != 0)
               sb.append(", ");
            QueueObject mqdlo = disListOfQNames.get(i);
            
            if (mqdlo.qMgrName != null)
               sb.append(mqdlo.qMgrName+" / "+mqdlo.qName);
            else
               sb.append(mqdlo.qName);
         }
         
         UFM.logger.info("     Distribution List    = "+sb.toString());
      }
      else
      {
         UFM.logger.info("     Queue Name           = " + qObject.qName);
         if (qObject.qMgrName != null)
            UFM.logger.info("     Remote QMgr Name     = " + qObject.qMgrName);
      }

      if (channelName != null)
      {
         UFM.logger.info("     Channel Name         = " + channelName);
         UFM.logger.info("     Hostname             = " + hostname);
         UFM.logger.info("     Port Number          = " + port);

         if (userID != null)
            UFM.logger.info("     UserID               = " + userID);

         if (password != null)
            UFM.logger.info("     Password             = *****");

         if (securityExit != null)
            UFM.logger.info("     SecurityExit         = " + securityExit);

         if (securityExitPath != null)
            UFM.logger.info("     SecurityExitPath     = " + securityExitPath);

         if (securityExitData != null)
            UFM.logger.info("     SecurityExitData     = " + securityExitData);

         if (cipherSuiteName != null)
            UFM.logger.info("     Cipher Suite Name    = " + cipherSuiteName);

         if (distinguishedName != null)
            UFM.logger.info("     Distinguished Name   = " + distinguishedName);

         if (trustedStore != null)
            UFM.logger.info("     Trusted Store        = " + trustedStore);

         if (trustedStorePasswd != null)
            UFM.logger.info("     Trusted StorePasswd  = " + trustedStorePasswd);

         if (keyStore != null)
            UFM.logger.info("     Key Store            = " + keyStore);

         if (keyStorePasswd != null)
            UFM.logger.info("     Key Store Passwd     = " + keyStorePasswd);

         if (ldapServer != null)
         {
            UFM.logger.info("     LDAP Server          = " + ldapServer);
            UFM.logger.info("     LDAP ServerPort      = " + ldapServerPort);
         }

         if (System.getProperty("javax.net.ssl.trustStore") != null)
            UFM.logger.info("     javax.net.ssl.trustStore = " + System.getProperty("javax.net.ssl.trustStore"));

         if (System.getProperty("javax.net.ssl.trustStorePassword") != null)
            UFM.logger.info("     javax.net.ssl.trustStorePassword = " + System.getProperty("javax.net.ssl.trustStorePassword"));

         if (System.getProperty("javax.net.ssl.keyStore") != null)
            UFM.logger.info("     javax.net.ssl.keyStore = " + System.getProperty("javax.net.ssl.keyStore"));

         if (System.getProperty("javax.net.ssl.keyStore") != null)
            UFM.logger.info("     javax.net.ssl.keyStorePassword = " + System.getProperty("javax.net.ssl.keyStorePassword"));
      }
      else if (CCDTFile != null)
      {
         UFM.logger.info("     CCDT File            = " + CCDTFile);

         if (userID != null)
            UFM.logger.info("     UserID               = " + userID);

         if (password != null)
            UFM.logger.info("     Password             = *****");
      }
   }
}
