package com.capitalware.ufm.xml;

import java.util.*;
import java.io.*;

import org.jdom.*;
import org.jdom.input.*;

import com.capitalware.ufm.objects.CWConstants;
import com.capitalware.ufm.schedule.Holiday;
import com.capitalware.ufm.xml.DTDs.ResolveLocalDTD;

/**
 * This class handles the reading and parsing of Holiday XML file and parse parameters
 *
 * @author Roger Lacroix
 * @version 1.0
 * @license Apache 2 License
 */
public class HolidayXml extends UFMXmlAbstract
{
   private Holiday[]                   _holidays;

   /**
    * The constructor
    * @param logger
    */
   public HolidayXml()
   {
      super();
   }

   /**
    * Read Holiday XML file and set parameters
    *
    * @param xmlFilename
    *           Filename of test_holiday.xml file
    * @throws IOException
    */
   public void readXMLFile(String xmlFilename) throws JDOMException, IOException
   {
      Holiday ho = null;
      Element dayElement = null;
      Element monthElement = null;
      Element yearElement = null;
      
      SAXBuilder builder = new SAXBuilder(true);
      // handle the ability to pull DTD from Jar if needed
      builder.setEntityResolver(new ResolveLocalDTD(CWConstants.HOLIDAY_DTD));
      Document doc = builder.build(CWConstants.SCHEDULE_PATH + xmlFilename);
      Element rootElement = doc.getRootElement();
      
      // read and parse holidays
      List holidayList = rootElement.getChildren(CWConstants.HOLIDAY_ELEMENT);
      _holidays = new Holiday[holidayList.size()];
      int si = 0;
      Iterator it = holidayList.iterator();
      while (it.hasNext())
      {
         Element holidayElement = (Element) it.next();
         ho = new Holiday();

         if (holidayElement.getAttribute(CWConstants.NAME_ATTRIBUTE) != null)
             ho.name = holidayElement.getAttribute(CWConstants.NAME_ATTRIBUTE).getValue().trim();

         if ( (dayElement = holidayElement.getChild(CWConstants.DAY_ELEMENT)) != null)
            ho.day = ho.formatStringToInt(dayElement.getTextTrim());

         if ( (monthElement = holidayElement.getChild(CWConstants.MONTH_ELEMENT)) != null)
            ho.month = ho.formatMonthToInts(monthElement.getTextTrim());
         
         if ( (yearElement = holidayElement.getChild(CWConstants.YEAR_ELEMENT)) != null)
            ho.year = ho.formatStringToInt(yearElement.getTextTrim());
         
         _holidays[si++] = ho;
      }
   }

   /**
    * Retrieve an array of holidays.
    * @return _holidays
    */
   public Holiday[] getHolidays()
   {
      return _holidays;
   }
}
